@@ -41,6 +41,9 @@ class Collaborator():
     def void_method(self):
         pass
     
+    def method_one(self, arg1):
+        return 1
+    
     alias_method = one_arg_method
 
 
@@ -234,7 +237,17 @@ class ProxySpyTests(unittest.TestCase):
             self.fail("not raised")
         except SomeException:
             pass            
+
+    def test_stub_returning_what_receives(self):
+        when(self.spy.method_one).then_return_input()
         
+        self.assertEquals(20, self.spy.method_one(20))
+
+    def test_stub_returning_what_receives_when_no_params(self):
+        when(self.spy.hello).then_return_input()
+        
+        self.failUnlessRaises(ApiMismatch, self.spy.hello)
+
     def test_be_able_to_return_objects(self):
         when(self.spy.one_arg_method).then_return(Collaborator())
         
@@ -267,7 +280,19 @@ class ProxySpyTests(unittest.TestCase):
         
         self.spy.alias_method(1)
         assert_that_was_called(self.spy.one_arg_method
-                               ).with_args(1)                
+                               ).with_args(1)
+                               
+    def test_was_never_called(self):
+        assert_that(self.spy.one_arg_method).was_never_called()
+
+    def test_was_never_called_is_false(self):
+        self.spy.one_arg_method(1)
+        try:
+             assert_that(self.spy.one_arg_method).was_never_called()
+             self.fail("it was called indeed!")
+        except UnexpectedBehavior:
+            pass
+                        
     
 class MethodPoolTests(unittest.TestCase):
     
@@ -291,6 +316,7 @@ class SpyTests(unittest.TestCase):
     def setUp(self):
         self.spy = spy(Collaborator())
         
+        
     def test_override_original_method(self):
         self.assertTrue(self.spy.hello() is None)
         
@@ -347,7 +373,7 @@ class SpyTests(unittest.TestCase):
     def test_assert_was_called_with_method_not_in_the_api(self):
         self.failUnlessRaises(ApiMismatch,
             assert_that_was_called, self.spy.unexisting_method)
-        
+       
         
 class MockTests(unittest.TestCase):
     
@@ -360,6 +386,15 @@ class MockTests(unittest.TestCase):
             self.fail('UnexpectedBehavior should be raised')
         except UnexpectedBehavior:
             pass
+
+    def test_fail_on_unexpected_call_msg_is_human_readable(self):
+        try:
+            self.mock.hello()
+        except UnexpectedBehavior, e:
+            for arg in e.args:
+                if re.search("No one", arg):
+                    return
+            self.fail("No enough readable exception message")        
         
     def test_define_expectation_and_call_method(self):
         expect_call(self.mock.hello)
@@ -605,6 +640,26 @@ class MatchersTests(unittest.TestCase):
         
         self.assertEquals(1000, 
                     self.spy.one_arg_method("abcde"))
+        
+    def test_matchers_when_passed_arg_is_none(self):
+        when(self.spy.one_arg_method).with_args(
+                        str_length(5)).then_return(1000)
+        self.assertTrue(self.spy.one_arg_method(None) is None)                        
+
+    def test_compare_objects_is_not_possible_without_eq_operator(self):
+        class SomeObject():
+            field1 = field2 = None
+            
+        obj = SomeObject()
+        obj2 = SomeObject()
+        self.spy.one_arg_method(obj)
+        
+        try:
+            assert_that(self.spy.one_arg_method).was_called().with_args(obj2)
+            self.fail('they should not match')
+        except ArgsDontMatch:
+            pass    
+                                
 
     
 if __name__ == "__main__":
